#include <stdlib.h>
#include <libporty/util/buf.h>
#include "sockev.h"
#include "filters.h"

/* init data is a string with list of separators (\r\n if NULL) */
void sockev_filter_split(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie)
{
	const char *default_sep = "\n\r";
	P_buf_t *b;
	char *line;
	int l;

	switch(event) {
		case SOCKEV_F_INIT:
			/* set up input buffer */
			*sockdata = P_buf_init(data == NULL ? default_sep : data);
			break;

		case SOCKEV_F_IN:
			/* split data in lines and call next for each line */
			b = *sockdata;
			P_buf_fillbin(b, data, len);
			while((line = P_buf_getbin(b, &l)) != NULL) {
				sockev_filter_next(id, event, line, l, cookie);
				P_buf_free(line);
				/* callback may have closed the socket during sockev_filter_next() call */
				b = *sockdata;
				if (b == NULL)
					break;
			}
			break;

		case SOCKEV_F_OUT:
			/* pass on */
			sockev_filter_next(id, event, data, len, cookie);
			break;

		case SOCKEV_F_CLOSE:
			/* free buffer */
			b = *sockdata;
			P_buf_uninit(b);
			*sockdata = NULL; /* paranoia... */
			break;

		case SOCKEV_F_OPEN:
			/* nothing to do yet */
			break;
	}
}
