#include <stdlib.h>
#include <libporty/net/network.h>
#include "buffer.h"

typedef struct sock_s sock_t;

typedef enum {
	ST_FREE = 0,
	ST_INUSE,    /* in use, but not yet decided how */
	ST_LISTEN,   /* listening socket */
	ST_CONNECT,  /* connecting socket */
	ST_COMM,     /* already connected, communicating */
	ST_DEL,      /* scheduled to delete */
	ST_VIRTUAL   /* virtual socket - for filter trickery (i.e. sessions) */
} sockev_state_t;

typedef enum {
	SA_RD = 1,
	SA_WR = 2
} sockev_alive_t;

struct sock_s {
	sockev_state_t state;
	P_net_socket s, sw;
	sockev_hooks_t hooks;
	char alive; /* which directions are still alive - sockev_alive_t */
	char watch; /* which directions to poll for */
	sockev_flags_t flags;
	sockev_buffer_t wbuff; /* write buffer */
	int closing; /* 0 if close-hook is not yet called */

	P_ipv4_addr_t remote_ip;
	char *remote_host;
	int remote_port;

	/* filter chain */
	int filters;                 /* number of filters */
	sockev_filter_cb *filt_cb;   /* array of callbacks */
	void **filt_data;            /* filter data */

	/* final read and write functions */
	void (*final_out)(sockev_t id, void **sockdata, sockev_f_dir_t event, const void *data, int len, int cookie);
	void (*final_in)(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie);
	void (*final_shutdown)(sockev_t id, int direction);
	void *final_data;

	void *userdata;
};

typedef struct sockev_ctx_s {
	struct P_pollfd *pfds;
	sock_t *socks;
	int alloced;
	int used;
	int grow;
	int in_poll;
} sockev_ctx_t;

extern sockev_ctx_t sockev_ctx;

#define PFDS  sockev_ctx.pfds
#define SOCKS sockev_ctx.socks
#define USED  sockev_ctx.used
#define ALLOCED  sockev_ctx.alloced
#define GROW  sockev_ctx.grow

#define wrong_id(id, expected_state) (((id) < 0) || ((id) > USED) || ((expected_state >= 0) && (SOCKS[id].state != expected_state)))

sockev_t sockev_alloc(void);
void sockev_del(sockev_t id);

/* set fds of a socket; if fdo is not P_NET_INVALID_SOCKET, there will be separate read socket (.s) and write socket (.sw) */
void sockev_setfds(sockev_t id, P_net_socket fdi, P_net_socket fdo);

/* Set events for poll depending on what we watch for and what directions are still open */
void sockev_setpoll(sockev_t id);


/* these are not in filters.h because they are called internally and never by filters */
void sockev_filter_final_out(sockev_t id, void **sockdata, sockev_f_dir_t event, const void *data, int len, int cookie);
void sockev_filter_final_in(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie);
